#!/bin/bash


####################################global params#################################
DEBOOTSTRAP_PATH=/mnt/BigDisk/workspace/precise/rootfs/debootstrap

ROOTSTOCK_PATH=/mnt/BigDisk/workspace/precise/rootfs/rootstock

KERNEL_RELEASE=3.2.46-100
#####################################################

function check_sudo()
{
	if [ `whoami` != root ]; then
		echo "this utility must be run from root or using sudo!"
		exit 1
	fi
}

function chroot_with_param()
{	
	local CHROOT_PATH=$1
	local CHROOT_SCRIPT=$2
	if [ x${CHROOT_PATH}  = x ]; then
		echo "usage $0 <chroot_path> [script]"
		exit 1
	fi
	#echo "CHROOT_PATH=${CHROOT_PATH}, CHROOT_SCRIPT=${CHROOT_SCRIPT}"

	mount -v --bind /dev ${CHROOT_PATH}/dev
	mount -vt devpts devpts  ${CHROOT_PATH}/dev/pts
	mount -vt tmpfs shm ${CHROOT_PATH}/dev/shm
	mount -vt proc proc ${CHROOT_PATH}/proc
	mount -vt sysfs sysfs ${CHROOT_PATH}/sys
	if [ x"${CHROOT_SCRIPT}" = x ]; then
		chroot ${CHROOT_PATH}  /usr/bin/env -i HOME=/root TERM="$TERM" PS1='\u:\w\$' PATH=/bin:/usr/bin:/sbin:/usr/sbin /bin/bash --login +h
	else
		chroot ${CHROOT_PATH}  /usr/bin/env -i HOME=/root TERM="$TERM" PS1='\u:\w\$' PATH=/bin:/usr/bin:/sbin:/usr/sbin /bin/bash -c "${CHROOT_SCRIPT}" --login +h
	fi

	umount -v ${CHROOT_PATH}/sys
	umount -v ${CHROOT_PATH}/proc
	umount -v ${CHROOT_PATH}/dev/shm
	umount -v ${CHROOT_PATH}/dev/pts
	umount -v ${CHROOT_PATH}/dev
}

function compile_kernel()
{
	local COMPILE_PATH=${DEBOOTSTRAP_PATH}/rootfs
	local COMPILE_SCRIPT="/arm-build.sh"
	chroot_with_param ${COMPILE_PATH} ${COMPILE_SCRIPT}
}

function create_image_from_scratch()
{
	####################################
	### let us create a new qemu image from scratch!##############
	BACKUP_BOOT=${ROOTSTOCK_PATH}/backup-boot
	BACKUP_ROOTFS=${ROOTSTOCK_PATH}/backup-rootfs
	SD_IMG=${ROOTSTOCK_PATH}/sd.img
	### The size is of unit of M 
	IMG_SIZE=2123
	ARM_CREATE_IMG_SCRIPT=${ROOTSTOCK_PATH}/arm-create-img.sh
	######################################################


	############first umount possible mount before rm the image file###################

	umount ${ROOTSTOCK_PATH}/boot
	umount ${ROOTSTOCK_PATH}/rootfs

	rm ${SD_IMG}

	${ARM_CREATE_IMG_SCRIPT} ${SD_IMG} ${IMG_SIZE} ${BACKUP_BOOT} ${BACKUP_ROOTFS}
	### need to change the owner so that it can run from user mode?
	##chown nick:nick ${SD_IMG}
	######## NEED TO MOUNT THE NEW IMAGE##################

	cd ${ROOTSTOCK_PATH};
	./arm-mount.sh
	#####################################################
}


#########now copy finished kernel file and create initrd for uboot##################
function copy_kernel()
{
	MOD_INSTALL_PATH=lib/modules

	ROOTFS=rootfs

	BOOT=boot

	SRC_PATH=usr/src/ubuntu-precise

	cp -r ${DEBOOTSTRAP_PATH}/${ROOTFS}/${MOD_INSTALL_PATH}/${KERNEL_RELEASE} ${ROOTSTOCK_PATH}/${ROOTFS}/${MOD_INSTALL_PATH}

	cp -v ${DEBOOTSTRAP_PATH}/${ROOTFS}/${SRC_PATH}/.config ${ROOTSTOCK_PATH}/${ROOTFS}/boot/config-${KERNEL_RELEASE}

	#cp -v ${DEBOOTSTRAP_PATH}/${ROOTFS}/${SRC_PATH}/.config  ${DEBOOTSTRAP_PATH}/boot/config-`date +"%Y%m%d%H%M%S"`

	cp -v ${DEBOOTSTRAP_PATH}/${ROOTFS}/${SRC_PATH}/arch/arm/boot/uImage ${ROOTSTOCK_PATH}/${ROOTFS}/boot/uImage-${KERNEL_RELEASE}

	cp -v ${DEBOOTSTRAP_PATH}/${ROOTFS}/${SRC_PATH}/arch/arm/boot/uImage ${ROOTSTOCK_PATH}/${BOOT}/uImage
}

#########################chroot and exec a script to create uInitrd############################
function create_uInitrd()
{
	local INITRD_PATH=${ROOTSTOCK_PATH}/rootfs
	local INITRD_SCRIPT="/arm-initrd.sh ${KERNEL_RELEASE}"

	chroot_with_param ${INITRD_PATH} "${INITRD_SCRIPT}"
	
	cp -v ${INITRD_PATH}/boot/uInitrd-${KERNEL_RELEASE}  ${INITRD_PATH}/boot/uInitrd
}

############### now run the qemu###########
function run_qemu()
{
	cd ${ROOTSTOCK_PATH}
	./arm-qemu.sh
}


set +e
check_sudo
compile_kernel
create_image_from_scratch
copy_kernel
create_uInitrd
run_qemu
set -e

